package net.xinhuamm.xycloud.xinhuamm_xycloud

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.xinhuamm.client.bridge.data.ShareData
import net.xinhuamm.xycloud.xinhuamm_xycloud.XYCloudManager.invokeFlutterMethod


/**
 * @author ming
 * @since 2024/1/8
 * for:分享弹窗
 */
class ShareDialog(private val shareData: ShareData) : BaseBottomDialog() {

    override fun getContentLayoutId(): Int {
        return R.layout.xinhuamm_xycloud_layout_dialog_share
    }

    override fun getBackgroundColorRes(): Int {
        return R.color.white
    }

    /**
     * 内容背景Drawable
     */
    override fun getGradientDrawable(): GradientDrawable {
        val shape = GradientDrawable()
//        val corner10 = dpToPx(10.0f)
//        shape.cornerRadii = floatArrayOf(corner10, corner10, corner10, corner10, 0f, 0f, 0f, 0f)
        shape.setColor(ContextCompat.getColor(context, backgroundColorRes))
        return shape
    }

    override fun initWidget(savedInstanceState: Bundle?) {
        super.initWidget(savedInstanceState)
        val listView = findViewById<HorizontalListView>(R.id.lv_share)
        val data: List<Int> = listOf(0, 1, 2, 3, 4)
        val adapter = HorizontalListAdapter(context, data)
        listView.adapter = adapter
        listView.setOnItemClickListener { _, _, index, _ ->
            val shareMap = ClassToMap.objectToMap(shareData)
            shareMap["platform"] = data[index].toString()
            invokeFlutterMethod("share", shareMap, null)
            dismiss()
        }
        findViewById<TextView>(R.id.tv_cancel).setOnClickListener { dismiss() }
    }

    class HorizontalListAdapter(context: Context?, private val data: List<Int>) : BaseAdapter() {
        private val inflater: LayoutInflater

        init {
            inflater = LayoutInflater.from(context)
        }

        override fun getCount(): Int {
            return data.size
        }

        override fun getItem(position: Int): Any {
            return data[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, child: View?, parent: ViewGroup?): View {
            var convertView: View? = child
            val holder: ViewHolder
            if (convertView == null) {
                convertView =
                    inflater.inflate(R.layout.xinhuamm_xycloud_layout_share_item, parent, false)
                holder = ViewHolder(convertView)
                convertView?.tag = holder
            } else {
                holder = convertView.tag as ViewHolder
            }
            val item = data[position]
            // 在这里设置列表项的内容
            holder.bind(item)
            return convertView!!
        }

        internal class ViewHolder(itemView: View) {
            // 在这里声明列表项中的视图元素
            var logo: ImageView
            var name: TextView

            init {
                logo = itemView.findViewById(R.id.iv_share_logo)
                name = itemView.findViewById(R.id.tv_share_name)
            }

            fun bind(item: Int) {
                // 在这里绑定数据到视图元素
                when (item) {
                    ShareType.WEIXIN_CIRCLE.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_by_firends_circle)
                        name.text = "朋友圈"
                    }
                    ShareType.WEIXIN.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_by_wechat)
                        name.text = "微信"
                    }
                    ShareType.WEIBO.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_weibo)
                        name.text = "微博"
                    }
                    ShareType.QQ.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_qq)
                        name.text = "QQ"
                    }
                    ShareType.QQ_ZONE.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_qq_zone)
                        name.text = "QQ空间"
                    }
                    ShareType.DD.intValue -> {
                        logo.setImageResource(R.drawable.xinhuamm_xycloud_icon_share_dd)
                        name.text = "钉钉"
                    }
                }
            }
        }
    }

}