package net.xinhuamm.xycloud.xinhuamm_xycloud

import android.content.Context
import android.os.Handler
import android.os.Looper
import io.flutter.plugin.common.MethodChannel

/**
 * @author ming
 * @since 2023/10/9
 * for:
 */
object XYCloudManager {

    // 宜昌开始支持新直播使用SDK打开
    const val FLAG_XINHUAMM_LIVE = "xhmm-live-h5"

    var channel: MethodChannel? = null

    var cloudDebug: Boolean = false
    var themeColor: String = "#FF5722"
    var userAgent: String = "xinhuamm"
    private var notifyVisibility: ((Boolean) -> Unit)? = null // 在XYCloudWebView内部初始化
    private var notifyBackPressed: (() -> Boolean)? = null

    fun openCloudActivity(context: Context, url: String) {
        XYCloudActivity.start(context, url)
    }

    fun setCloudIsDebug(flag: Boolean) {
        cloudDebug = flag
    }

    fun setCloudThemeColor(themeColor: String) {
        XYCloudManager.themeColor = themeColor
    }

    fun setCloudUserAgent(userAgent: String) {
        XYCloudManager.userAgent = userAgent
    }

    fun isLogin(callback: (boolean: Boolean) -> Unit) {
        getUserId {
            callback.invoke(it.isNotEmpty())
        }
    }

    fun notifyBackPressed(): Boolean {
        return notifyBackPressed?.invoke() == true
    }

    fun setNotifyBackPressed(notify: () -> Boolean) {
        notifyBackPressed = notify
    }

    fun setNotifyVisibility(notify: (Boolean) -> Unit) {
        notifyVisibility = notify
    }

    fun notifyVisibility(isVisible: Boolean) {
        notifyVisibility?.invoke(isVisible)
    }

    fun getUserId(callback: (userId: String) -> Unit) {
        getUserInfo {
            callback.invoke(it?.get("id") ?: "")
        }
    }

    fun getUserInfo(callback: (loginParams: Map<String, String>?) -> Unit) {
        invokeFlutterMethod("getUserInfo", null, object : MethodChannel.Result {
            override fun success(result: Any?) {
                if (result is HashMap<*, *>) {
                    val map = result as HashMap<String, String>
                    if (map.containsKey("userId")) {
                        // userId 转换为 id
                        map["id"] = map["userId"]!!
                    }
                    callback.invoke(map)
                } else {
                    callback.invoke(null)
                }
            }

            override fun error(errorCode: String, errorMessage: String?, errorDetails: Any?) {
                callback.invoke(null)
            }

            override fun notImplemented() {

            }
        })
    }

    fun getToken(callback: (loginParams: Map<String, String>?) -> Unit) {
        invokeFlutterMethod("getToken", null, object : MethodChannel.Result {
            override fun success(result: Any?) {
                if (result is Map<*, *>) {
                    callback.invoke(result as Map<String, String>)
                } else {
                    callback.invoke(null)
                }
            }

            override fun error(errorCode: String, errorMessage: String?, errorDetails: Any?) {
                callback.invoke(null)
            }

            override fun notImplemented() {

            }
        })
    }

    fun login(callback: MethodChannel.Result? = null) {
        invokeFlutterMethod("login", null, callback)
    }

    fun setMethodChannel(channel: MethodChannel) {
        XYCloudManager.channel = channel
    }

    fun invokeFlutterMethod(
        method: String,
        arguments: Any? = null,
        callback: MethodChannel.Result? = null
    ) {
        Handler(Looper.getMainLooper()).post {
            channel?.invokeMethod(method, arguments, callback)
        }
    }
}