package net.xinhuamm.xycloud.xinhuamm_xycloud

import android.content.Context
import android.graphics.Color
import android.net.UrlQuerySanitizer
import android.text.TextUtils
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import androidx.fragment.app.FragmentActivity
import io.flutter.plugin.platform.PlatformView
import java.util.*

/**
 * author : yuntong
 * date   : 2023/11/17 14:32
 * desc   :
 */
class XYCloudWebView(
    val context: FragmentActivity,
    val viewId: Int,
    val args: Any?
) : PlatformView {

    private var rootView: FrameLayout = FrameLayout(context)
    private var fragment: XYCloudFragment? = null

    init {
        rootView.layoutParams = ViewGroup.LayoutParams(
            ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT
        )
        if (args is Map<*, *> && args["url"] is String) {
            val url = args["url"] as String
            val params = getUrlParameters(url)
            val showStatusBar = params["showStatusBar"] == "1"

            val container = if (showStatusBar) {
                val linear = LinearLayout(context)
                linear.orientation = LinearLayout.VERTICAL
                linear.layoutParams = ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT
                )
                rootView.addView(linear)

                val status = View(context)
                status.setBackgroundColor(Color.WHITE)
                status.layoutParams = ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, getStatusBarHeight(context)
                )
                linear.addView(status)

                val fragmentContainer = FrameLayout(context)
                fragmentContainer.layoutParams = ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT
                )
                linear.addView(fragmentContainer)

                fragmentContainer
            } else {
                rootView
            }

            addFragment(container, url)
        } else {
            rootView.addView(XYCloudErrorView(context, args).view)
        }
    }

    override fun getView(): View {
        return rootView
    }

    override fun dispose() {
        fragment?.onDestroy()
    }

    private fun addFragment(view: View, url: String) {
        view.id = R.id.xinhuamm_xycloud_id_container
        fragment = XYCloudFragment.newInstance(url)
        val fm = context.supportFragmentManager
        fm.beginTransaction()
            .replace(view.id, fragment!!)
            .commitAllowingStateLoss()
        XYCloudManager.setNotifyVisibility {
            if (it) {
                fragment?.onResume()
            } else {
                fragment?.onPause()
            }
        }
        XYCloudManager.setNotifyBackPressed {
            fragment?.onBackPressed() == true
        }
    }

    private fun getUrlParameters(url: String?): HashMap<String, String> {
        val paramsMap = HashMap<String, String>()
        val sanitizer = UrlQuerySanitizer()
        sanitizer.allowUnregisteredParamaters = true
        sanitizer.unregisteredParameterValueSanitizer = UrlQuerySanitizer.getAllButNulLegal()
        sanitizer.parseUrl(url ?: "")
        val query = sanitizer.parameterSet
        if (!query.isEmpty()) {
            val treeQuery = TreeSet(query)
            for (key in treeQuery) {
                val value = sanitizer.getValue(key)
                paramsMap[key] = value
            }
        }
        return paramsMap
    }

    /**
     * 获取状态栏高度
     *
     * @param context context
     * @return 状态栏高度
     */
    private fun getStatusBarHeight(context: Context): Int {
        // 获得状态栏高度
        val resourceId = context.resources.getIdentifier("status_bar_height", "dimen", "android")
        return context.resources.getDimensionPixelSize(resourceId)
    }

}