//
//  XinhuammHelper.m
//  xinhuamm_xycloud
//
//  Created by 邓付磊 on 2024/1/8.
//

#import "XinhuammHelper.h"

@implementation XinhuammHelper


/// 从xinhuamm.bundle获取image
/// - Parameter imageName: 图片名称
+ (UIImage *)imageNamed:(NSString *)imageName {
    NSString *path = [[NSBundle mainBundle] pathForResource:@"XYO2O" ofType:@"bundle"];
    NSBundle *bundle =  [NSBundle bundleWithPath:path];
    if(!bundle) {
        return [UIImage imageNamed:imageName];
    }
    UIImage *image  = [UIImage imageNamed:imageName inBundle:bundle compatibleWithTraitCollection:nil];
    
    if(!image) {
        image = [UIImage imageNamed:imageName];
    }
    return image;
   
}

/// keyWindow
+ (UIWindow *)keyWindow {
    UIWindow *keyWindow = nil;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 130000 // 编译时检查SDK版本（兼容不同版本的Xcode，防止编译报错）
    if (@available(iOS 13.0, *)) { // 运行时检查系统版本（兼容不同版本的系统，防止运行报错）
        NSSet<UIScene *> *connectedScenes = [UIApplication sharedApplication].connectedScenes;
        for (UIScene *scene in connectedScenes) {
            if (scene.activationState == UISceneActivationStateForegroundActive && [scene isKindOfClass:[UIWindowScene class]]) {
                UIWindowScene *windowScene = (UIWindowScene *)scene;
                for (UIWindow *window in windowScene.windows) {
                    if (window.isKeyWindow) {
                        keyWindow = window;
                        break;
                    }
                }
            }
        }
    }
#endif
        
    if (!keyWindow) {
        keyWindow = [UIApplication sharedApplication].windows.firstObject;
        if (!keyWindow.isKeyWindow) {
#if __IPHONE_OS_VERSION_MIN_REQUIRED < 130000
            UIWindow *window = [UIApplication sharedApplication].keyWindow;
            if (CGRectEqualToRect(window.bounds, UIScreen.mainScreen.bounds)) {
                keyWindow = window;
            }
#endif
        }
    }
    
    return keyWindow;
}

/// 获取UINavigationController
+ (UINavigationController *)mainNavigation {
    return [self currentController].navigationController;
}

/// 获取当前controller
+ (UIViewController *)currentController {
    UIWindow *keyWindow = [self keyWindow];
    UIViewController *visibleViewController = [self xyCurrentVisibleController:keyWindow];
    return visibleViewController;
}

+ (CGFloat)safeAreaHeight {
    static dispatch_once_t onceToken;
    static CGFloat safeAreaHeight = NO;
    if (![UIApplication sharedApplication].windows){
        // 在+load方法里调用会出现这种情况
        // 我这里直接返回了0，其实如果需要在+load方法里判断是否是圆角，只能通过机型列表或者分辨率等方式判断，每次发布新机型都需要更新方法。
        // 所以该方法在+load里调用无效
        return 0;
    } else {
        dispatch_once(&onceToken, ^{
            if (@available(iOS 11.0, *)) {
                UIWindow *window = [[UIWindow alloc] initWithFrame:[UIScreen mainScreen].bounds];
                safeAreaHeight = window.safeAreaInsets.bottom;
            }
        });
        return safeAreaHeight;
    }
}

/// 获取状态栏的高度
+ (CGFloat)statusBarHeight {
    float statusBarHeight = 0;
    if (@available(iOS 13.0, *)) {
        UIStatusBarManager *statusBarManager = [UIApplication sharedApplication].windows.firstObject.windowScene.statusBarManager;
        statusBarHeight = statusBarManager.statusBarFrame.size.height;
    } else {
        #pragma clang diagnostic push
        #pragma clang diagnostic ignored "-Wdeprecated-declarations"
        statusBarHeight = [UIApplication sharedApplication].statusBarFrame.size.height;
        #pragma clang diagnostic pop
    }
    if (statusBarHeight==0) {
        statusBarHeight = CGRectGetHeight([[UIApplication sharedApplication] statusBarFrame]);
    }
    
    if (statusBarHeight==0) {
        statusBarHeight = [self safeAreaHeight] > 0 ? 44 : 20;
    }
    return statusBarHeight;
}

/// 当前应用名称
+ (NSString *)getAppName {
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    NSString *appCurName = [infoDictionary objectForKey:@"CFBundleDisplayName"];
    return appCurName?:@"";
}


#pragma mark - private
+ (UIViewController *)xyCurrentVisibleController:(UIWindow *)window {
    UIViewController *rootViewController = window.rootViewController;
    return [self getVisibleViewControllerFrom:rootViewController];
}

+ (UIViewController *)getVisibleViewControllerFrom:(UIViewController *)vc {
    if ([vc isKindOfClass:[UINavigationController class]]) {
        return [self getVisibleViewControllerFrom:[((UINavigationController *) vc) topViewController]];
        
    } else if ([vc isKindOfClass:[UITabBarController class]]) {
        return [self getVisibleViewControllerFrom:[((UITabBarController *) vc) selectedViewController]];
        
    } else {
        if (vc.presentedViewController && ![vc.presentedViewController isKindOfClass:[UIAlertController class]]) {
            return [self getVisibleViewControllerFrom:vc.presentedViewController];
        } else {
            return vc;
        }
    }
}
@end
