#import "XinhuammXycloudPlugin.h"
#import "XYCloudManager.h"
#import <YYModel/YYModel.h>
#import "XYCloudPlatformViewFactory.h"
#import "XYCloudGlobalConfig.h"
#import "XinhuammHelper.h"
#import "XYShareView.h"

/// 打开 Cloud page
NSString *const kOpenCloudPageMethodName = @"openCloudPage";
/// 全局配置
NSString *const kSetCloudGlobalConfigMethodName = @"setCloudGlobalConfig";
/// 获取token
NSString *const  kGetTokenMethodName = @"getToken";
/// 获取用户信息
NSString *const kGetUserInfoMethodName = @"getUserInfo";
/// 分享
NSString *const kShareMethodName = @"share";
///  登录
NSString *const kLoginMethodName = @"login";
/// 打开扫一扫
NSString *const kOpenScanPageMethodName = @"openScanPage";
/// 处理扫码结果
NSString *const kHandleScanResultMethodName = @"handleScanResult";
/// 调用显示隐藏（主要是安卓端处理H5页面显示隐藏）
NSString *const kNotifyCloudPageVisibilityMethodName = @"notifyCloudPageVisibility";


@interface XinhuammXycloudPlugin()

///  通信相关类
@property (nonatomic, strong) FlutterMethodChannel* channel;

@end

@implementation XinhuammXycloudPlugin

+ (instancetype)instance {
    static dispatch_once_t onceToken;
    static XinhuammXycloudPlugin *sInstance;
    dispatch_once(&onceToken, ^{
        sInstance = [[XinhuammXycloudPlugin alloc] init];
    });
    return sInstance;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"xinhuamm_xycloud"
                                                                binaryMessenger:[registrar messenger]];
    XinhuammXycloudPlugin *instance = [XinhuammXycloudPlugin instance];
    instance.channel = channel;
    [registrar addMethodCallDelegate:instance channel:channel];
    
    XYCloudPlatformViewFactory *factory = [[XYCloudPlatformViewFactory alloc] init];
    [registrar registerViewFactory:factory withId:@"XYCloudFlutterWebView"];
}

/// 监听Flutter端调用原生端事件
/// - Parameters:
///   - call: flutter事件参数包含方法名 method、方法参数arguments
///   - result: flutter端回调
- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([kSetCloudGlobalConfigMethodName isEqualToString:call.method]) {
        if ([call.arguments isKindOfClass:[NSDictionary class]]) {
            /// 全局配置
            NSDictionary *dict = call.arguments;
            XYCloudGlobalConfig *model = [XYCloudGlobalConfig yy_modelWithDictionary:dict];
            if (model.weixinSchemeUrl.length) {
                [XYCloudGlobalConfig shareInstance].weixinSchemeUrl = model.weixinSchemeUrl;
            }
            if (model.appUrlScheme.length) {
                [XYCloudGlobalConfig shareInstance].appUrlScheme = model.appUrlScheme;
            }
            if (model.themeColor.length) {
                [XYCloudGlobalConfig shareInstance].themeColor = model.themeColor;
            }
            if (model.userAgent.length) {
                [XYCloudGlobalConfig shareInstance].userAgent = model.userAgent;
            }
            if (model.navigationColor.length) {
                [XYCloudGlobalConfig shareInstance].navigationColor = model.navigationColor;
            }
            
        }
        result([@"iOS " stringByAppendingString:call.method]);
        return;
    }
    
    if ([kOpenCloudPageMethodName isEqualToString:call.method]) {
        XYCloudManager *manager = [XYCloudManager managerWithUrl:call.arguments];
        [manager enterO2OPageFromController:nil isPush:YES];
        result([@"iOS " stringByAppendingString:call.method]);
        return;
    }
    if ([kOpenScanPageMethodName isEqualToString:call.method]) {
        [XYO2OModule openScanPage:^(NSString * _Nonnull scanResult) {
            //延 diss完后处理
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.8 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                if ([scanResult hasPrefix:@"020"]) {
                    NSString *handleResult = [XYO2OModule getUrlStringWithScanResult:scanResult];
                    XYCloudManager *manager = [XYCloudManager managerWithUrl:handleResult];
                    [manager enterO2OPageFromController:nil isPush:NO];
                } else if([XYO2OModule isSDKUrl:scanResult]) {
                    XYCloudManager *manager = [XYCloudManager managerWithUrl:scanResult];
                    [manager enterO2OPageFromController:nil isPush:NO];
                }
               
                result(scanResult);
            });
        }];
        return;
    }
    if ([kHandleScanResultMethodName isEqualToString:call.method]) {
        NSString *arguments = [NSString stringWithFormat:@"%@",call.arguments];
        if ([arguments hasPrefix:@"020"] || [XYO2OModule isSDKUrl:arguments]) {
            NSString *handleResult = [XYO2OModule getUrlStringWithScanResult:arguments];
            XYCloudManager *manager = [XYCloudManager managerWithUrl:handleResult];
            [manager enterO2OPageFromController:nil isPush:NO];
            result(@(YES));
            return;
        }
        if ([XYO2OModule isSDKUrl:arguments]) {
            XYCloudManager *manager = [XYCloudManager managerWithUrl:arguments];
            [manager enterO2OPageFromController:nil isPush:NO];
            result(@(YES));
            return;
        }
        result(@(NO));
        return;
    }
    
    if ([kNotifyCloudPageVisibilityMethodName isEqualToString:call.method]) {
        //        BOOL visibility = [call.arguments boolValue];
        /// 安卓端需要处理相关逻辑，iOS端暂不需要处理（这里实现该方法是为了flutter测异常）
        result([@"iOS " stringByAppendingString:call.method]);
        return;
    }
    result(FlutterMethodNotImplemented);
}

/// 获取token
- (void)getCloudTokenWith:(NSDictionary *)parameters block:(void (^)(NSDictionary *))block {
    
    [self invokeMethod:kGetTokenMethodName arguments:nil result:^(id  _Nullable result) {
        if (!result)  return;
        //        XYCloudGlobalConfig *model = [XYCloudGlobalConfig yy_modelWithDictionary:result];
        //        if (!model.appUserToken.length) return;
        //        [XYCloudGlobalConfig shareInstance].appUserToken = model.appUserToken;
        //        /// 测试数据
        //        NSMutableDictionary *data = [NSMutableDictionary dictionary];
        //        data[@"status"] = @"200";
        //        NSMutableDictionary *msg = [NSMutableDictionary dictionary];
        //        /// params 为token
        //        msg[@"params"] = model.appUserToken;
        block ? block(result) : nil;
    }];
}

/// 获取用户信息
- (void)getUserInfoWith:(NSDictionary *)parameters block:(void (^)(NSDictionary *))block {
    BOOL goLogin = NO;
    if ([parameters.allKeys containsObject:@"goLogin"]) {
        goLogin = [parameters[@"goLogin"] boolValue];
    }
    if(goLogin) {
        /// 拉起登录页
        [self invokeMethod:kLoginMethodName arguments:nil result:^(id  _Nullable result) {
            if (!result)  return;
            NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithDictionary:result];
            if ([[dict allKeys] containsObject:@"userId"]) {
                /// 需要参数id
                dict[@"id"] = result[@"userId"];
            }
            block ? block(dict) : nil;
        }];
        return;
    }
    [self invokeMethod:kGetUserInfoMethodName arguments:nil result:^(id  _Nullable result) {
        if (!result)  return;
        NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithDictionary:result];
        if ([[dict allKeys] containsObject:@"userId"]) {
            /// 需要参数id
            dict[@"id"] = result[@"userId"];
        }
        block ? block(dict) : nil;
    }];
}

/// 分享
- (void)shareWithParameter:(NSDictionary *)shareDic getPosterBlock:(void (^)(BOOL))getPosterBlock {
    XYShareView *view = [[XYShareView alloc] initWithFrame:[UIScreen mainScreen].bounds];
    //    UIWindow *window = [[UIApplication sharedApplication].delegate window];
    UIWindow *window = [XinhuammHelper keyWindow];
    [window addSubview:view];
    view.shareToPlatformBlock = ^(NSInteger platform) {
        NSMutableDictionary *dic = [NSMutableDictionary dictionaryWithDictionary:shareDic];
        dic[@"platform"] = @(platform).stringValue;
        [self invokeMethod:kShareMethodName arguments:dic result:nil];
    };
}

#pragma mark -privite
///  原生端主动调用Flutter
/// - Parameters:
///   - method: 方法名
///   - arguments: 参数
///   - callback: 回调
- (void)invokeMethod:(NSString *)method arguments:(id)arguments result:(FlutterResult)callback {
    [self.channel invokeMethod:method arguments:arguments result:callback];
}

@end
